--
-- PowerShaft
-- Specialization for PowerShaft
--
-- @author  	Manuel Leithner (SFM-Modding)
-- @version 	v2.0
-- @date  		15/10/10
-- @history:	v1.0 - Initial version
--				v2.0 - added network support, changed update to updateTick
--

PowerShaft = {};

function PowerShaft.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(ArticulatedAxis, specializations);
end;

function PowerShaft:load(xmlFile)
	
	local shaftCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.drivingPowerShafts#count"), 0);
    self.drivingPowerShafts = {};
    for i=1, shaftCount do
		local shaft = {};
        local shaftName = string.format("vehicle.drivingPowerShafts.powerShaft%d", i);
		shaft.node = Utils.indexToObject(self.components, getXMLString(xmlFile, shaftName .. "#index"));		
		shaft.trans = Utils.getNoNil(getXMLFloat(xmlFile, shaftName .. "#translation"), 0);
		local x,y,z = getTranslation(getParent(shaft.node));
		shaft.orgTrans = {x,y,z};
		shaft.rot = Utils.degToRad(Utils.getNoNil(getXMLFloat(xmlFile, shaftName .. "#rotation"), 0));	
		shaft.speedFactor = Utils.getNoNil(getXMLFloat(xmlFile, shaftName .. "#speedFactor"), 2);
		table.insert(self.drivingPowerShafts, shaft);        
    end;
end;

function PowerShaft:delete()
end;

function PowerShaft:readStream(streamId, connection)
end;

function PowerShaft:writeStream(streamId, connection)
end;

function PowerShaft:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PowerShaft:keyEvent(unicode, sym, modifier, isDown)
end;

function PowerShaft:update(dt)
end;

function PowerShaft:updateTick(dt)

	if self:getIsActive() then
		local steeringAngle;
		if self.isServer then
			steeringAngle = self.articulatedAxis.curRot;
		else
			steeringAngle = self.rotatedTime * self.articulatedAxis.rotSpeed;
			if steeringAngle > self.articulatedAxis.rotMax then
				steeringAngle = self.articulatedAxis.rotMax;
			elseif steeringAngle < self.articulatedAxis.rotMin then
				steeringAngle = self.articulatedAxis.rotMin;
			end;
		end;
	
		for _, shaft in pairs(self.drivingPowerShafts) do
			local x,y,z = getRotation(self.wheels[1].driveNode);
			setRotation(shaft.node, 0 ,0, x*shaft.speedFactor);

			if shaft.trans ~= 0 then
				local newTrans = shaft.trans * steeringAngle/self.articulatedAxis.rotMax;
				local x,y,z = unpack(shaft.orgTrans);
				setTranslation(getParent(shaft.node), x + newTrans,y,z);
			end;
			if shaft.rot ~= 0 then
				newRot = shaft.rot * steeringAngle/self.articulatedAxis.rotMax;
				setRotation(getParent(shaft.node), 0, newRot, 0);
			end;			
		end;		
	end;
end;

function PowerShaft:draw()	
end;


